@@ -4,10 +4,25 @@
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2015 Gerd Neugebauer
+** (c) 1996-2016 Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
+** This program is free software; you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation; either version 2, or (at your option)
+** any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program; if not, write to the Free Software
+** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+**
+**-----------------------------------------------------------------------------
 ** >>> See the file doc/c_lib.tex for a printed documentation of the
 ** >>> main parts of the C source.
 ** 
@@ -63,6 +78,7 @@
 #include <bibtool/sbuffer.h>
 #include <bibtool/expand.h>
 #include <bibtool/crossref.h>
+#include <bibtool/io.h>
 #include <bibtool/version.h>
 #ifdef HAVE_LIBKPATHSEA
 #ifdef __STDC__
@@ -91,9 +107,6 @@
  static int rec_lt_cased _ARG((Record r1,Record r2));/* main.c               */
  static int update_crossref _ARG((DB db,Record rec));/* main.c               */
  static void usage _ARG((int fullp));		   /* main.c                 */
- void save_input_file _ARG((char *file));	   /* main.c                 */
- void save_macro_file _ARG((char *file));	   /* main.c                 */
- void save_output_file _ARG((char * file));	   /* main.c                 */
 
 /*****************************************************************************/
 /* External Programs and Variables					     */
@@ -127,8 +140,8 @@ char * getenv(name)				   /*			     */
     "\t%cA<c>\t\tKind of disambiguating key strings: <c>=0|a|A\n",
     "\t%cc\t\tInclude crossreferenced entries into the output (toggle)\n",
     "\t%cd\t\tCheck double entries (toggle)\n",
-    "\t%cf <format>\tKey generation enabled (formated key)\n",
-    "\t%cF\t\tKey generation enabled with formated key\n",
+    "\t%cf <format>\tKey generation enabled (formatted key)\n",
+    "\t%cF\t\tKey generation enabled with formatted key\n",
     "\t%ch\t\tPrint this help info and exit\n",
     "\t[%ci] infile\tSpecify input file. If %ci omitted it may not start\n",
     "\t\t\twith a %c. If absent stdin is taken to read from.\n",
@@ -155,7 +168,7 @@ char * getenv(name)				   /*			     */
     "\t%c$\t\tSymbol table output (debugging only)\n",
 #endif
     0L,
-    "Copyright (C) 2014 Gerd Neugebauer",
+    "Copyright (C) 2016 Gerd Neugebauer",
     "gene@gerd-neugebauer.de"
   };
 
@@ -217,7 +230,6 @@ static void usage(fullp)			   /*			     */
   ErrPrintF("%s\n",SPECIAL_OPTIONS);		   /*                        */
 }						   /*------------------------*/
 
-
 #define UnknownWarning(X)     WARNING2("Unknown flag ignored: ",X)
 #define NoSFileWarning	      WARNING("Missing select option. Flag ignored.")
 #define NoFileError(X)	      WARNING3("File ",X," not found.")
@@ -225,109 +237,6 @@ static void usage(fullp)			   /*			     */
 #define MissingResource	      WARNING("Missing resource.")
 #define NoRscError(X)	      WARNING3("Resource file ",X," not found.")
 
-/*****************************************************************************/
-/***			 Input File Pipe Section			   ***/
-/*****************************************************************************/
-
-#define InputFilePipeIncrement 8
-
- static char **input_files;
- static int  input_file_size = 0;
- static int  input_file_ptr  = 0;
-
-#define InputPipeIsFull		(input_file_ptr >= input_file_size)
-#define InputPipeIsEmpty	(input_file_ptr == 0)
-#define PushToInputPipe(FILE)	input_files[input_file_ptr++] = FILE
-#define ForAllInputFiles(FILE)	for (FILE=input_files;			\
-				     FILE<&input_files[input_file_ptr];	\
-				     FILE++)
-
-/*-----------------------------------------------------------------------------
-** Function:	save_input_file()
-** Purpose:	The input file pipe is a dynamic array of strings.
-**		This fifo stack is used to store the input \BibTeX{}
-**		files to be processed by \BibTool.
-**
-**		This function is called to push an string into the pipe.
-**		If necessary the array has to be allocated or enlarged.
-**		This is done in larger junks to avoid lots of calls to
-**		|realloc()|.
-** Arguments:
-**	file	File name to save.
-** Returns:	nothing
-**___________________________________________________			     */
-void save_input_file(file)			   /*			     */
-  char *file;					   /*			     */
-{						   /*			     */
-  if ( file == NULL )				   /*			     */
-  { WARNING("Missing input file name. Flag ignored.");/*		     */
-    return;					   /*			     */
-  }						   /*			     */
-  if ( *file == '-' && file[1] == '\0' )	   /*			     */
-  { file = NULL; }				   /*			     */
-						   /*			     */
-  if ( InputPipeIsFull )			   /* No space left?	     */
-  { input_file_size += InputFilePipeIncrement;	   /*			     */
-						   /*			     */
-    if ( InputPipeIsEmpty			   /* Try to enlarge array   */
-	? NULL==(input_files=			   /*			     */
-		 (char**)malloc(sizeof(char*)	   /*			     */
-				*(size_t)input_file_size))/*		     */
-	: NULL==(input_files=			   /*			     */
-		 (char**)realloc((char*)input_files,/*			     */
-				 sizeof(char*)	   /*			     */
-				 *(size_t)input_file_size))/*		     */
-	)					   /*			     */
-    { OUT_OF_MEMORY("input file pipe.");	   /*		             */
-    }						   /*			     */
-  }						   /*			     */
-  PushToInputPipe(file);			   /*			     */
-}						   /*------------------------*/
-
-
-/*****************************************************************************/
-/***			   Output File Section				   ***/
-/*****************************************************************************/
-
- static char *output_file = NULL;		   /*			     */
-
-/*-----------------------------------------------------------------------------
-** Function:	save_output_file()
-** Purpose:	Simply feed the output file name into the static variable.
-**		This function is useful since it can be called from rsc.c
-** Arguments:
-**	file	File name to save
-** Returns:	nothing
-**___________________________________________________			     */
-void save_output_file(file)			   /*			     */
-  char * file;					   /*			     */
-{ if ( output_file != NULL )			   /*			     */
-  { WARNING2("Output file redefined: ",file); }	   /*			     */
-  output_file = file;				   /*			     */
-}						   /*------------------------*/
-
-
-/*****************************************************************************/
-/***			   Macro File Section				   ***/
-/*****************************************************************************/
-
- static char *macro_file = NULL;		   /*			     */
-
-/*-----------------------------------------------------------------------------
-** Function:	save_macro_file()
-** Purpose:	Simply feed the macro file name into the static variable.
-**		This function is useful since it can be called from rsc.c
-** Arguments:
-**	file	File name to save
-** Returns:	nothing
-**___________________________________________________			     */
-void save_macro_file(file)			   /*			     */
-  char *file;					   /*			     */
-{ if ( macro_file != NULL )			   /*			     */
-  { WARNING2("Macro file redefined: ",file); }	   /*			     */
-  macro_file = file;				   /*			     */
-}						   /*------------------------*/
-
 
 /*****************************************************************************/
 /***				    MAIN				   ***/
@@ -337,20 +246,19 @@ void save_macro_file(file)			   /*			     */
 /*-----------------------------------------------------------------------------
 ** Function:	keep_selected()
 ** Type:	static int
-** Purpose:	
+** Purpose:	Mark the record as deleted if it is not selected.
 **		
 ** Arguments:
-**	db	
-**	rec	
-** Returns:	
+**	db	the database
+**	rec	the record
+** Returns:	FALSE
 **___________________________________________________			     */
-static int keep_selected(db,rec)		   /*                        */
+static int keep_selected(db, rec)		   /*                        */
   DB db;					   /*                        */
   Record rec;					   /*                        */
 {						   /*                        */
-  if ( !is_selected(db,rec) )			   /*                        */
-  { SetRecordDELETED(rec);			   /*                        */
-  }						   /*                        */
+  if (!is_selected(db, rec))			   /*                        */
+  { SetRecordDELETED(rec); }			   /*                        */
  						   /*                        */
   return FALSE;					   /*                        */
 }						   /*------------------------*/
@@ -362,21 +270,21 @@ static int keep_selected(db,rec)		   /*                        */
 ** Purpose:	Undelete crossreferenced entries
 **		
 ** Arguments:
-**	db	
-**	rec	
-** Returns:	
+**	db	the database
+**	rec	the record
+** Returns:	FALSE
 **___________________________________________________			     */
 int keep_xref(db,rec)				   /*                        */
   DB db;					   /*                        */
   Record rec;					   /*                        */
 {						   /*                        */
   if ( !RecordIsDELETED(rec) )		   	   /*                        */
-  { Uchar  *key;				   /*                        */
+  { String key;				   	   /*                        */
     int    count;				   /*                        */
  						   /*                        */
-    for ( count = rsc_xref_limit;		   /* Prevent infinite loop  */
-	  count >= 0;		   	   	   /*                        */
-	  count-- )				   /*                        */
+    for (count = rsc_xref_limit;		   /* Prevent infinite loop  */
+	 count >= 0;		   	   	   /*                        */
+	 count--)				   /*                        */
     {					   	   /*                        */
       if ( (key = get_field(db,rec,sym_crossref)) == NULL )/*                */
       { return FALSE; }			   	   /*                        */
@@ -390,7 +298,7 @@ int keep_xref(db,rec)				   /*                        */
         return FALSE;			   	   /*                        */
       }					   	   /*                        */
  						   /*                        */
-      if ( !RecordIsDELETED(rec) ) { return FALSE; }/*                       */
+      if (!RecordIsDELETED(rec)) { return FALSE; } /*                        */
       ClearRecordDELETED(rec);		   	   /*                        */
     }					   	   /*                        */
  						   /*                        */
@@ -401,6 +309,53 @@ int keep_xref(db,rec)				   /*                        */
 }						   /*------------------------*/
 #endif
 
+/*-----------------------------------------------------------------------------
+** Function:	write_macros()
+** Type:	static void
+** Purpose:	
+**		
+** Arguments:
+**	m_file	the name of the macro file of NULL for none
+**	the_db	the database
+** Returns:	nothing
+**___________________________________________________			     */
+static void write_macros(m_file, the_db)	   /*                        */
+  char* m_file;		   			   /*                        */
+  DB	the_db;					   /*                        */
+{ FILE * file;					   /*                        */
+ 						   /*                        */
+  if ( m_file == NULL || *m_file == 0 ) return;    /*			     */
+ 						   /*                        */
+  if ( rsc_verbose )			   	   /*			     */
+  { VerbosePrint1("Writing macros"); }	   	   /*			     */
+					       	   /*                        */
+  if ( strcmp(m_file,"-") == 0 ||		   /*                        */
+       (file=fopen(m_file,"w")) == NULL )          /*                        */
+  { file = stdout; }			   	   /*                        */
+  print_db(file,the_db,rsc_all_macs?"s":"S");      /*                        */
+  if ( file != stdout ) { fclose(file); }	   /*                        */
+}						   /*------------------------*/
+
+/*-----------------------------------------------------------------------------
+** Function:	read_in_files()
+** Type:	static void
+** Purpose:	
+**		
+** Arguments:
+**	the_db	
+** Returns:	nothing
+**___________________________________________________			     */
+static void read_in_files(the_db)		   /*                        */
+  DB the_db;					   /*                        */
+{ int n = get_no_inputs();			   /*                        */
+  int i;					   /*                        */
+  for (i = 0; i < n; i++)		   	   /* For all input files    */
+  { if ( read_db(the_db,			   /*                        */
+		 (String)get_input_file(i),	   /*                        */
+		 rsc_verbose) )	   		   /*                        */
+    { NoFileError(get_input_file(i)); }		   /*			     */
+  }						   /*			     */
+}						   /*------------------------*/
 
 #define Toggle(X) X = !(X)
 
@@ -434,6 +389,7 @@ int main(argc,argv)				   /*			     */
   int	(*fct)();			   	   /* Function pointer	     */
   int	c_len;					   /*                        */
   int   *c = NULL;				   /*                        */
+  char  *o_file;				   /*                        */
  						   /*                        */
   init_error(stderr);				   /*                        */
   init_bibtool(argv[0]);			   /*                        */
@@ -445,7 +401,7 @@ int main(argc,argv)				   /*			     */
     }						   /*			     */
     else					   /*			     */
     { switch ( *++ap )				   /*			     */
-      { case 'A': set_base((Uchar*)(ap+1));  break;/* disambiguation	     */
+      { case 'A': set_base((String)(ap+1));  break;/* disambiguation	     */
 	case 'c': Toggle(rsc_xref_select);   break;/* include crossreferences*/
 	case 'd': Toggle(rsc_double_check);  break;/* double entries	     */
 #ifdef HAVE_LIBKPATHSEA
@@ -469,7 +425,7 @@ int main(argc,argv)				   /*			     */
 	  break; 				   /*			     */
 	case 'q': Toggle(rsc_quiet);	    break; /* quiet		     */
 	case 'r':				   /* resource file	     */
-	  if ( ++i < argc && load_rsc((Uchar*)(argv[i])) )/*		     */
+	  if ( ++i < argc && load_rsc((String)(argv[i])) )/*		     */
 	  { need_rsc = FALSE; }		   	   /*			     */
 	  else					   /*                        */
 	  {  NoRscError(argv[i]); }		   /*                        */
@@ -485,22 +441,23 @@ int main(argc,argv)				   /*			     */
 	case 'x':				   /* extract		     */
 	  rsc_all_macs = FALSE;			   /*                        */
 	  if ( ++i < argc )			   /*			     */
-	  { read_aux((Uchar*)(argv[i]),		   /*                        */
+	  { read_aux((String)(argv[i]),		   /*                        */
 		     save_input_file,		   /*                        */
 		     *++ap=='v');  		   /*                        */
 	  }					   /*                        */
-	  else		    { NoSFileWarning; }	   /*			     */
+	  else					   /*                        */
+	  { NoSFileWarning; }	   		   /*			     */
 	  break;				   /*			     */
 	case 'X':				   /* extract pattern	     */
 	  rsc_all_macs = FALSE;			   /*                        */
-	  if ( ++i < argc ) { save_regex((Uchar*)argv[i]); }/*		     */
+	  if ( ++i < argc ) { save_regex((String)argv[i]); }/*		     */
 	  else		    { MissingPattern; }	   /*			     */
 	  break;				   /*			     */
 	case '#': Toggle(rsc_cnt_all);	    break; /* print full statistics  */
 	case '@': Toggle(rsc_cnt_used);	    break; /* print short statistics */
 	case '-':				   /* extended command	     */
-	  if ( *++ap )	      { (void)use_rsc((Uchar*)ap); }/*		     */
-	  else if ( ++i<argc ){ (void)use_rsc((Uchar*)argv[i]); }/*	     */
+	  if ( *++ap )	      { (void)use_rsc((String)ap); }/*		     */
+	  else if ( ++i<argc ){ (void)use_rsc((String)argv[i]); }/*	     */
 	  else		      { MissingResource;  }/*			     */
 	  break;				   /*			     */
 #ifdef SYMBOL_DUMP
@@ -516,20 +473,14 @@ int main(argc,argv)				   /*			     */
 						   /*			     */
   if ( need_rsc ) { (void)search_rsc(); }	   /*			     */
 						   /*			     */
-  if ( InputPipeIsEmpty )			   /* If no input file given */
+  if ( get_no_inputs() == 0 )			   /* If no input file given */
   { save_input_file("-"); }			   /*  then read from stdin  */
 						   /*			     */
   init_read();					   /* Just in case the path  */
  						   /*  has been modified.    */
   the_db = new_db();				   /*                        */
 						   /*			     */
-  for ( i = 0; i < input_file_ptr; i++ )	   /* For all input files    */
-  {						   /*			     */
-    if ( read_db(the_db,			   /*                        */
-		 (Uchar*)(input_files[i]),	   /*                        */
-		 rsc_verbose) )	   		   /*                        */
-    { NoFileError(input_files[i]); }		   /*			     */
-  }						   /*			     */
+  read_in_files(the_db);			   /*                        */
  						   /*                        */
   db_forall(the_db,keep_selected);		   /*                        */
 						   /*                        */
@@ -548,7 +499,7 @@ int main(argc,argv)				   /*			     */
     }						   /*                        */
   }						   /*                        */
 						   /*			     */
-  if ( rsc_expand_crossref )			   /*                        */
+  if ( rsc_expand_crossref || rsc_expand_xdata )   /*                        */
   {						   /*			     */
     db_forall(the_db,expand_crossref);		   /*                        */
   }						   /*			     */
@@ -597,8 +548,10 @@ int main(argc,argv)				   /*			     */
   if ( rsc_double_check	)		   	   /* Maybe look for doubles */
   { db_forall(the_db,dbl_check); }		   /*                        */
  						   /*                        */
-  if ( output_file == NULL ||			   /*                        */
-       (file=fopen(output_file,"w")) == NULL )	   /*                        */
+  o_file = get_output_file();			   /*                        */
+ 						   /*                        */
+  if ( o_file == NULL ||			   /*                        */
+       (file=fopen(o_file,"w")) == NULL )	   /*                        */
   { file = stdout; }				   /*                        */
  						   /*                        */
   if ( rsc_select ) { rsc_del_q = FALSE; }	   /*                        */
@@ -620,16 +573,7 @@ int main(argc,argv)				   /*			     */
  						   /*                        */
   if ( file != stdout ) { fclose(file); }	   /*                        */
 						   /*			     */
-  if ( macro_file != NULL  && *macro_file )	   /*			     */
-  { if ( rsc_verbose )				   /*			     */
-    { VerbosePrint1("Writing macros"); }	   /*			     */
- 						   /*                        */
-    if ( strcmp(macro_file,"-") == 0 ||		   /*                        */
-	 (file=fopen(macro_file,"w")) == NULL )    /*                        */
-    { file = stdout; }				   /*                        */
-    print_db(file,the_db,rsc_all_macs?"s":"S");	   /*                        */
-    if ( file != stdout ) { fclose(file); }	   /*                        */
-  }						   /*			     */
+  write_macros(get_macro_file(), the_db);	   /*                        */
 						   /*			     */
   if ( rsc_cnt_all || rsc_cnt_used )		   /*			     */
   { int i;					   /*                        */
@@ -644,7 +588,7 @@ int main(argc,argv)				   /*			     */
 		   cnt[i]);		   	   /*			     */
       }						   /*			     */
     }						   /*			     */
-    free(c);
+    free(c);					   /*                        */
   }						   /*                        */
 						   /*			     */
 #ifdef SYMBOL_DUMP
@@ -672,7 +616,7 @@ int main(argc,argv)				   /*			     */
 **	r2
 ** Returns:	
 **___________________________________________________			     */
-static int rec_gt(r1,r2)			   /*                        */
+static int rec_gt(r1, r2)			   /*                        */
   Record r1;					   /*                        */
   Record r2;					   /*                        */
 { return (strcmp((char*)RecordSortkey(r1),	   /*                        */
@@ -689,7 +633,7 @@ static int rec_gt(r1,r2)			   /*                        */
 **	r2
 ** Returns:	
 **___________________________________________________			     */
-static int rec_lt(r1,r2)			   /*                        */
+static int rec_lt(r1, r2)			   /*                        */
   Record r1;					   /*                        */
   Record r2;					   /*                        */
 { return (strcmp((char*)RecordSortkey(r1),	   /*                        */
@@ -706,7 +650,7 @@ static int rec_lt(r1,r2)			   /*                        */
 **	r2	
 ** Returns:	
 **___________________________________________________			     */
-static int rec_gt_cased(r1,r2)			   /*                        */
+static int rec_gt_cased(r1, r2)			   /*                        */
   Record r1;					   /*                        */
   Record r2;					   /*                        */
 { return (strcmp((char*)get_key_name(RecordSortkey(r1)),/*                   */
@@ -723,7 +667,7 @@ static int rec_gt_cased(r1,r2)			   /*                        */
 **	r2	
 ** Returns:	
 **___________________________________________________			     */
-static int rec_lt_cased(r1,r2)			   /*                        */
+static int rec_lt_cased(r1, r2)			   /*                        */
   Record r1;					   /*                        */
   Record r2;					   /*                        */
 { return (strcmp((char*)get_key_name(RecordSortkey(r1)),/*                   */
@@ -739,7 +683,7 @@ static int rec_lt_cased(r1,r2)			   /*                        */
 **	rec
 ** Returns:	
 **___________________________________________________			     */
-static int do_keys(db,rec)			   /*                        */
+static int do_keys(db, rec)			   /*                        */
   DB	 db;					   /*                        */
   Record rec;					   /*                        */
 {						   /*                        */
@@ -760,7 +704,7 @@ static int do_keys(db,rec)			   /*                        */
 **	rec
 ** Returns:	
 **___________________________________________________			     */
-static int do_no_keys(db,rec)			   /*                        */
+static int do_no_keys(db, rec)			   /*                        */
   DB	 db;					   /*                        */
   Record rec;					   /*                        */
 {						   /*                        */
@@ -782,18 +726,18 @@ static int do_no_keys(db,rec)			   /*                        */
 **	first_rec
 ** Returns:	nothing
 **___________________________________________________			     */
-static int update_crossref(db,rec)		   /*			     */
+static int update_crossref(db, rec)		   /*			     */
   DB		 db;				   /*                        */
   Record	 rec;				   /*			     */
-{ register Uchar **hp;				   /*			     */
+{ register String *hp;				   /*			     */
   register int   i;				   /*                        */
-  Uchar		 *t,*s;			   	   /*			     */
+  String	 s, t;			   	   /*			     */
 						   /*			     */
   if ( !RecordIsXREF(rec) ) return 0;		   /*			     */
 						   /*                        */
-  for ( i = RecordFree(rec), hp = RecordHeap(rec); /* search crossref field  */
-	i > 0 && *hp != sym_crossref;		   /*			     */
-	i -= 2, hp += 2 )			   /*			     */
+  for (i = RecordFree(rec), hp = RecordHeap(rec);  /* search crossref field  */
+       i > 0 && *hp != sym_crossref;		   /*			     */
+       i -= 2, hp += 2)				   /*			     */
   { }						   /*			     */
   if ( i <= 0 )					   /*			     */
   { DebugPrint1("*** No crossref found.");	   /*			     */
@@ -802,15 +746,15 @@ static int update_crossref(db,rec)		   /*			     */
 						   /*			     */
   t = *++hp; t++;				   /*			     */
   (void)sp_open(t);				   /* Try to extract	     */
-  if ( (s = SParseSymbol(&t)) == (Uchar*)0 )	   /*  the crossref as symbol*/
+  if ( (s = SParseSymbol(&t)) == StringNULL )	   /*  the crossref as symbol*/
   { return 0; }					   /*			     */
 						   /*			     */
-  if ( (s = db_new_key(db,s)) == (Uchar*)0 )	   /*			     */
+  if ( (s = db_new_key(db,s)) == StringNULL )	   /*			     */
   { ERROR2("Crossref not found: ",(char*)s);	   /*			     */
     return 0;					   /*			     */
   }						   /*			     */
   if (rsc_key_case) { s = get_key_name(s); }	   /*                        */
-  if ( (t=(Uchar*)malloc(strlen((char*)s)+3))==(Uchar*)NULL )/* get temp mem */
+  if ( (t=(String)malloc(strlen((char*)s)+3))==(String)NULL )/* get temp mem */
   { OUT_OF_MEMORY("update_crossref()"); }	   /*		             */
 						   /*			     */
   (void)sprintf((char*)t,(**hp=='"'?"\"%s\"":"{%s}"),s);/* make new crossref */
@@ -819,7 +763,6 @@ static int update_crossref(db,rec)		   /*			     */
   return 0;					   /*                        */
 }						   /*------------------------*/
 
-
 #define equal_records(R1,R2) RecordSortkey(R1) == RecordSortkey(R2)
 
 /*-----------------------------------------------------------------------------
@@ -831,7 +774,7 @@ static int update_crossref(db,rec)		   /*			     */
 **	rec
 ** Returns:	
 **___________________________________________________			     */
-static int dbl_check(db,rec)			   /*                        */
+static int dbl_check(db, rec)			   /*                        */
   DB	 db;					   /*                        */
   Record rec;					   /*                        */
 {						   /*                        */
@@ -839,11 +782,11 @@ static int dbl_check(db,rec)			   /*                        */
        && equal_records(PrevRecord(rec),rec) )	   /*			     */
   {						   /*                        */
     if ( !rsc_quiet )				   /*                        */
-    { Uchar *k1 = *RecordHeap(rec);		   /*                        */
-      Uchar *k2 = *RecordHeap(PrevRecord(rec));	   /*                        */
+    { String k1 = *RecordHeap(rec);		   /*                        */
+      String k2 = *RecordHeap(PrevRecord(rec));	   /*                        */
       ErrPrint("*** BibTool WARNING: Possible double entries discovered: \n***\t");
-      if ( k1 == NULL ) k1 = (Uchar*) "";	   /*                        */
-      if ( k2 == NULL ) k2 = (Uchar*) "";	   /*                        */
+      if ( k1 == NULL ) k1 = s_empty;	   	   /*                        */
+      if ( k2 == NULL ) k2 = s_empty;	   	   /*                        */
       ErrPrint((char*)k2);			   /*                        */
       ErrPrint(" =?= ");			   /*                        */
       ErrPrint((char*)k1);			   /*                        */
